#ifndef POKEPLATINUM_OV26_02256404_H
#define POKEPLATINUM_OV26_02256404_H

#include "struct_decls/struct_02018340_decl.h"
#include "overlay026/struct_ov26_02256404_1.h"
#include "overlay026/struct_ov26_02256404_decl.h"

BOOL ov26_02256404(UnkStruct_ov26_02256404 ** param0, const UnkStruct_ov26_02256404_1 * param1, UnkStruct_02018340 * param2);
void ov26_0225649C(UnkStruct_ov26_02256404 * param0);
void ov26_022564A8(UnkStruct_ov26_02256404 * param0, u32 param1);
BOOL ov26_022564CC(UnkStruct_ov26_02256404 * param0, u32 param1);
BOOL ov26_022564D8(UnkStruct_ov26_02256404 * param0);

#endif // POKEPLATINUM_OV26_02256404_H
